module Specific
  module Command
    class Feature
      attr_reader :spec, :group_name, :file_name, :title

      def initialize(spec, group_name, file_name, title)
        @spec = spec
        @group_name = group_name
        @file_name = file_name
        @title = title
      end

      def run
        mk_group_dir
        write_feature_template
      end

      private

      def mk_group_dir
        FileUtils.mkdir_p group_dir_path
      end

      def sanitized_group_name
        group_name.gsub(' ', '-').downcase
      end

      def group_dir_path
        dir_name = sanitized_group_name
        File.join spec.base_path, dir_name
      end

      def feature_file_path
        File.join(group_dir_path, file_name + ".feature")
      end

      def write_feature_template
        template = <<-EOT
          @id_[ID] @group_[GROUP]
          Feature: [NAME]
            In Order
            As
            I

            Scenario:
              Given
              When
              Then
        EOT
        template_intendation = template.match(/^(\s*)/)[1]
        template.gsub!(/^#{template_intendation}/, '')

        subs = {
          'ID' => next_id,
          'GROUP' => sanitized_group_name,
          'NAME' => title,
        }

        subs.each do |key, value|
          template.gsub!(/\[#{key}\]/, value.to_s)
        end

        if File.exist?(feature_file_path)
          raise "File #{printable_feature_file_path} already exists"
        end

        File.open(feature_file_path, 'w')do |f|
          f << template
        end
        puts "File #{printable_feature_file_path} created"
      end

      def next_id
        spec.next_id
      end

      def printable_feature_file_path
        feature_file_path.gsub(/^#{Dir.pwd}\//, '')
      end
    end
  end
end
