require 'gherkin_ruby'

module Specific
  module Processor
    class Gherkin
      def initialize(spec)
        @spec = spec
      end

      def execute(ast)
        ast.accept(self)
      end

      def visit_Feature(ast_feature)
        tags = ast_feature.tags.map do |ast_tag|
          Specific::Tag.new(ast_tag.name)
        end

        scenarios = ast_feature.scenarios.map do |ast_scenario|
          steps = ast_scenario.steps.map do |ast_step|
            Specific::Step.new(ast_step.keyword, ast_step.name)
          end
          Specific::Scenario.new(ast_scenario.name, steps)
        end

        feature = Specific::Feature.new(
          ast_feature.name, 
          tags, 
          ast_feature.description,
          scenarios
        )
        @spec.add_feature feature
      end
    end
  end
end
