require 'set'

module Specific
  module Renderer

    def self.for(type)
      renderer = Base.registered.detect do |renderer_class|
        renderer_class.types.include? type
      end 

      renderer || raise("No Renderer for #{type.inspect} found")
    end

    module Base
      def self.registered
        @registered ||= Set.new
      end
      def self.included(cls)
        Base.registered << cls
        cls.send :extend, ClassMethods
      end

      module ClassMethods
        def types(*types)
          @types = types unless types.empty?
          @types || []
        end
      end
    end
  end
end

require 'specific/renderer/text'
require 'specific/renderer/markdown'
